@@ -19,6 +19,8 @@
 #include <plasma/plasma_export.h> // for PLASMA_ENABLE_DEPRECATED_SINCE
 #include <plasma_version.h>
 
+#include <memory>
+
 #include "krunner_export.h"
 #include "querymatch.h"
 #include "runnercontext.h"
@@ -82,7 +84,7 @@ public:
     /** An ordered list of runners */
     typedef QList<AbstractRunner *> List;
 
-    virtual ~AbstractRunner();
+    ~AbstractRunner() override;
 
     /**
      * This is the main query method. It should trigger creation of
@@ -439,6 +441,15 @@ protected:
     friend class RunnerManager;
     friend class RunnerManagerPrivate;
 
+    /**
+     * Constructor for a KRunner plugin
+     * @param parent parent object for this runner
+     * @param pluginMetaData metadata that was embedded in the runner
+     * @param args for compatibility with KPluginFactory, should be passed on to the parent constructor
+     * @since 5.72
+     */
+    AbstractRunner(QObject *parent, const KPluginMetaData &pluginMetaData, const QVariantList &args);
+
 #if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 77)
     KRUNNER_DEPRECATED_VERSION(5, 77, "use AbstractRunner(QObject *, const KPluginMetaData &, const QVariantList &)")
     explicit AbstractRunner(QObject *parent = nullptr, const QString &path = QString());
@@ -453,10 +464,10 @@ protected:
     explicit AbstractRunner(const KService::Ptr service, QObject *parent = nullptr);
 #endif
 #endif
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 86)
     /// @since 5.72
     explicit AbstractRunner(const KPluginMetaData &pluginMetaData, QObject *parent = nullptr);
-    /// @since 5.77
-    AbstractRunner(QObject *parent, const KPluginMetaData &pluginMetaData, const QVariantList &args);
+#endif
 #if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 77)
     /// @deprecated Since 5.77, use AbstractRunner(QObject *, const KPluginMetaData &, const QVariantList &)
     KRUNNER_DEPRECATED_VERSION(5, 77, "use AbstractRunner(QObject *, const KPluginMetaData &, const QVariantList &)")
@@ -515,15 +526,30 @@ protected:
      */
     virtual QList<QAction *> actionsForMatch(const Plasma::QueryMatch &match);
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * Creates and then adds an action to the action registry.
      * AbstractRunner assumes ownership of the created action.
      *
+     * @see QueryMatch::setActions
+     * @see actionsForMatch
+     *
      * @param id A unique identifier string
      * @param icon The icon to display
      * @param text The text to display
      * @return the created QAction
+     * @deprecated Since 5.86 create the QAction instance manually
+     * @code
+     * // in the runner class definition
+     * QList<QAction *> m_actions;
+     * // when initializing the runner, optionally set the date if needed
+     * auto action = new QAction(QIcon::fromTheme(iconName), text, this);
+     * m_actions << action;
+     * // when creating the match
+     * match.setActions(m_actions);
+     * @endcode
      */
+    KRUNNER_DEPRECATED_VERSION(5, 86, "create the QAction instance manually")
     QAction *addAction(const QString &id, const QIcon &icon, const QString &text);
 
     /**
@@ -534,7 +560,9 @@ protected:
      *
      * @param id A unique identifier string
      * @param action The QAction to be stored
+     * @deprecated Since 5.86, create the QAction instance manually
      */
+    KRUNNER_DEPRECATED_VERSION(5, 86, "create the QAction instance manually")
     void addAction(const QString &id, QAction *action);
 
     /**
@@ -542,24 +570,33 @@ protected:
      * AbstractRunner deletes the action once removed.
      *
      * @param id The id of the action to be removed
+     * @deprecated Since 5.86, deprecated for lack of usage
      */
+    KRUNNER_DEPRECATED_VERSION(5, 86, "deprecated for lack of usage")
     void removeAction(const QString &id);
 
     /**
      * Returns the action associated with the id
+     * @deprecated Since 5.86, create the QAction instances manually and store them in a custom container instead
      */
+    KRUNNER_DEPRECATED_VERSION(5, 86, "create the QAction instances manually and store them in a custom container instead")
     QAction *action(const QString &id) const;
 
     /**
      * Returns all registered actions
+     * @deprecated Since 5.86, create the QAction instances manually and store them in a custom container instead
      */
+    KRUNNER_DEPRECATED_VERSION(5, 86, "create the QAction instances manually and store them in a custom container instead")
     QHash<QString, QAction *> actions() const;
 
     /**
      * Clears the action registry.
      * The action pool deletes the actions.
+     * @deprecated Since 5.86, use a custom container to store the QAction instances instead
      */
+    KRUNNER_DEPRECATED_VERSION(5, 86, "use a custom container to store the QAction instances instead")
     void clearActions();
+#endif
 
     /**
      * Adds a registered syntax that this runner understands. This is used to
@@ -640,7 +677,7 @@ protected:
     virtual QMimeData *mimeDataForMatch(const Plasma::QueryMatch &match);
 
 private:
-    AbstractRunnerPrivate *const d;
+    std::unique_ptr<AbstractRunnerPrivate> const d;
     bool hasUniqueResults();
     bool hasWeakResults();
     friend class RunnerContext;
@@ -652,7 +689,7 @@ private:
 
 // clang-format off
 #if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 72)
-// Boilerplate to emit a version-controlled warning about the deprecated macro at leats with GCC
+// Boilerplate to emit a version-controlled warning about the deprecated macro at least with GCC
 #if KRUNNER_DEPRECATED_WARNINGS_SINCE >= 0x054800 // 5.72.0
 #   if defined(__GNUC__)
 #       define K_EXPORT_PLASMA_RUNNER_DO_PRAGMA(x) _Pragma (#x)
@@ -672,10 +709,10 @@ private:
  */
 #define K_EXPORT_PLASMA_RUNNER( libname, classname )     \
 K_EXPORT_PLASMA_RUNNER_WARNING("Deprecated. Since 5.72, use K_EXPORT_PLASMA_RUNNER_WITH_JSON(classname, jsonFile) instead") \
-K_PLUGIN_FACTORY(factory, registerPlugin<classname>();) \
-K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+K_PLUGIN_FACTORY(factory, registerPlugin<classname>();)
 #endif
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 88)
 /**
  * @relates Plasma::AbstractRunner
  *
@@ -685,10 +722,11 @@ K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
  * @param jsonFile name of the JSON file to be compiled into the plugin as metadata
  *
  * @since 5.72
+ * @deprecated Since 5.88 use K_PLUGIN_CLASS_WITH_JSON instead
  */
 #define K_EXPORT_PLASMA_RUNNER_WITH_JSON(classname, jsonFile) \
-    K_PLUGIN_FACTORY_WITH_JSON(classname ## Factory, jsonFile, registerPlugin<classname>();) \
-    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+    K_PLUGIN_FACTORY_WITH_JSON(classname ## Factory, jsonFile, registerPlugin<classname>();)
+#endif
 
 #if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 75)
 /**
@@ -698,8 +736,7 @@ K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
  * @deprecated Since 5.0, use K_PLUGIN_FACTORY directly
  */
 #define K_EXPORT_RUNNER_CONFIG( name, classname )     \
-K_PLUGIN_FACTORY(ConfigFactory, registerPlugin<classname>();) \
-K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+K_PLUGIN_FACTORY(ConfigFactory, registerPlugin<classname>();)
 #endif
 // clang-format on
 
